@@ -14,7 +14,6 @@ local fetch = require("luarocks.fetch")
 local pack = require("luarocks.pack")
 local remove = require("luarocks.remove")
 local deps = require("luarocks.deps")
-local manif = require("luarocks.manif")
 
 util.add_run_function(make)
 make.help_summary = "Compile package in current directory using a rockspec."
@@ -33,9 +32,6 @@ This command is useful as a tool for debugging rockspecs.
 To install rocks, you'll normally want to use the "install" and
 "build" commands. See the help on those for details.
 
-NB: Use `luarocks install` with the `--only-deps` flag if you want to install
-only dependencies of the rockspec (see `luarocks help install`).
-
 --pack-binary-rock  Do not install rock. Instead, produce a .rock file
                     with the contents of compilation in the current
                     directory.
@@ -81,13 +77,10 @@ function make.command(flags, rockspec)
       ok, err = build.build_rockspec(rockspec, false, true, deps.get_deps_mode(flags))
       if not ok then return nil, err end
       local name, version = ok, err
-
       if (not flags["keep"]) and not cfg.keep_other_versions then
          local ok, err = remove.remove_other_versions(name, version, flags["force"], flags["force-fast"])
          if not ok then util.printerr(err) end
       end
-
-      manif.check_dependencies(nil, deps.get_deps_mode(flags))
       return name, version
    end
 end
