@@ -3,7 +3,6 @@
 -- saving tables into files.
 -- Implemented separately to avoid interdependencies,
 -- as it is used in the bootstrapping stage of the cfg module.
---module("luarocks.persist", package.seeall)
 local persist = {}
 package.loaded["luarocks.persist"] = persist
 
@@ -147,7 +146,7 @@ write_table = function(out, tbl, level, field_order)
       end
 
       write_value(out, v, level, sub_order)
-      if type(k) == "number" then
+      if type(v) == "number" then
          sep = ", "
          indent = false
       else
